const scope   = require('./scope');
const utils   = require('./utils');
const finder  = require('./utils/interactionFinder');
const Interaction   = require('./Interaction');
const pointerEvents = require('./pointerEvents/base');
const Eventable = require('./Eventable');
//require('./legacyBrowsers');
require('./touchToMouse');

const eventables = [];

pointerEvents.signals.on('collect-targets', function ({ targets, element, eventType }) {
  const eventable = element._eventable;

  if (eventable && eventable[eventType]) {
    targets.push({
      element: this,
      eventable: eventable,
    });
  }
});

//const returnTrue = ()=>true;
//const returnFalse = ()=>false;

// make it like a jQuery event to avoid duplication
pointerEvents.signals.on('new', function ({ pointerEvent }) {
  pointerEvent._fromInteract = true;

  // would need to do this afte rthe jquery patching or whatever
  if (pointerEvent.pointerType === 'touch') {
    //pointerEvent.preventDefault();
  }
/*
  pointerEvent[jQuery.expando] = true;

  pointerEvent.isPropagationStopped = jQuery.Event.prototype.isPropagationStopped;
  pointerEvent.isImmediatePropagationStopped = jQuery.Event.prototype.isImmediatePropagationStopped;
  pointerEvent.isDefaultPrevented = (pointerEvent.defaultPrevented
                             || pointerEvent.defaultPrevented === undefined
                             && pointerEvent.returnValue === false ? returnTrue : returnFalse);

  pointerEvent.preventDefault = jQuery.Event.prototype.preventDefault;
  pointerEvent.stopPropagation = function () {
    jQuery.Event.prototype.stopPropagation.call(this);
    pointerEvent.propagationStopped = true;
  };

  pointerEvent.stopImmediatePropagation = function () {
    jQuery.Event.prototype.stopImmediatePropagation.call(this);
    pointerEvent.immediatePropagationStopped = true;
  };
*/
});

function createTypeFixer (type) {
  return function (event) {
    event._originalType = event.type;
    event.type = type;
  };
}

function restoreType (event) {
  event.type = event._originalType;
}

const jQuery = window.jQuery;

function createSpecial (bindType, eventType) {
  return {
    setup: function (data, namespaces, eventHandle) {
      //window.console.log('settingup', bindType, eventType, this);

      scope.addDocument(scope.getWindow(this).document);

      let eventable = this._eventable;

      if (!eventable) {
        eventable = this._eventable = new Eventable();
        eventable._element = this;
        eventable._count = 0;

        eventables.push(eventable);
      }

      const handlers = eventable[eventType] = (eventable[eventType] || []);

      eventable._count++;

      handlers.push(createTypeFixer(bindType));
      handlers.push(eventHandle);
      handlers.push(restoreType);
    },

    // needs fixing
    teardown: function () {
      const index = eventables.indexOf(this._eventable);
      const eventable = eventables[index];

      if (!eventable._count) {
        eventables.splice(eventable, 1);
        this._eventable = null;
        window.console.log('DOING TEARDOWN!!!');
      }
    },

    /*
    handle: function (event)  {
      event.type = bindType;
      const ret = event.handleObj.handler.apply(this, arguments);
      event.type = event.handleObj.origType;

      return ret;
    },
   */
  };
}

for (const type of [ 'down', 'move', 'up', 'cancel', 'tap', 'doubletap', 'hold' ]) {
  jQuery.event.special[type] = createSpecial(type, type);
}

const mouseToPointer = {
  mousedown: 'down',
  click    : 'tap',
  mousemove: 'move',
  mouseup  : 'up',
  dblclick : 'doubletap',
};

for (const type in mouseToPointer) {
  const orignSpecial = jQuery.event.special[type];

  const special = createSpecial(type, mouseToPointer[type]);
  utils.extend(special, orignSpecial);

  jQuery.event.special[type] = special;
}

//HTMLElement.prototype.click = function () {
  //const interaction = finder.mouse({ mouseEvent: true, eventType: 'click' }) || new Interaction(true);

  //pointerEvents.firePointers(

module.exports = {
  scope,
  utils,
  Interaction,
  pointerEvents,
  events: require('./utils/events'),
};

pointerEvents.signals.on('new', arg=>{
  window.console.log(arg.event.type, arg.pointerEvent.type);
});

['out', 'over', 'enter', 'leave'].forEach(type=> document.addEventListener('mouse' + type, e=>console.log(e.type, e.target.classList)));

window.console.log('Done setting up pointerEvents');
