const modifiers      = require('./index');
const utils          = require('../utils');
const defaultOptions = require('../defaultOptions');

const transform = {
  defaults: {
    enabled    : true,
    endOnly    : false,
    transformation: 0.5,
    elementRect: null,
  },

  setOffset: function () {},

  set: function ({ pageCoords, status, options }) {
    const transformation = options && options.transformation;

    if (!transformation) {
      return;
    }

    const page = status.useStatusXY
      ? { x: status.x, y: status.y }
      : utils.extend({}, pageCoords);

    status.dx = 0;
    status.dy = 0;
    status.locked = false;

    const modifiedX = page.x * transformation;
    const modifiedY = page.y * transformation;

    status.dx = modifiedX - page.x;
    status.dy = modifiedY - page.y;

    status.changed = status.modifiedX !== modifiedX || status.modifiedY !== modifiedY;
    status.locked = !!(status.dx || status.dy);

    status.modifiedX = modifiedX;
    status.modifiedY = modifiedY;
  },

  modifyCoords: function ({ page, client, status, phase, options }) {
    const elementRect = options && options.elementRect;

    if (options && options.enabled
        && !(phase === 'start' && elementRect && status.locked)) {

      if (status.locked) {
        page.x += status.dx;
        page.y += status.dy;
        client.x += status.dx;
        client.y += status.dy;
      }

      return {
        range : status.range,
        locked: status.locked,
        x     : status.modifiedX,
        y     : status.modifiedY,
        realX : status.realX,
        realY : status.realY,
        dx    : status.dx,
        dy    : status.dy,
      };
    }
  },
};

modifiers.transform = transform;
modifiers.names.push('transform');

defaultOptions.perAction.transform = transform.defaults;

module.exports = transform;
